@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2020 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,25 +37,26 @@ These are the terms:
 
 """
 
+import bleachbit
+from bleachbit import Command, FileUtilities, General
 
 import glob
 import logging
 import os
-import platform
 import re
 import sys
-import traceback
 
 from decimal import Decimal
 
 if 'win32' == sys.platform:
-    import _winreg
+    import winreg
     import pywintypes
     import win32api
     import win32con
     import win32file
     import win32gui
     import win32process
+    import win32security
 
     from ctypes import windll, c_ulong, c_buffer, byref, sizeof
     from win32com.shell import shell, shellcon
@@ -63,10 +64,7 @@ if 'win32' == sys.platform:
     psapi = windll.psapi
     kernel = windll.kernel32
 
-import Command
-import Common
-import FileUtilities
-import General
+logger = logging.getLogger(__name__)
 
 
 def browse_file(_, title):
@@ -77,7 +75,7 @@ def browse_file(_, title):
                                         | win32con.OFN_FILEMUSTEXIST
                                         | win32con.OFN_HIDEREADONLY,
                                         Title=title)
-    except pywintypes.error, e:
+    except pywintypes.error as e:
         logger = logging.getLogger(__name__)
         if 0 == e.winerror:
             logger.debug('browse_file(): user cancelled')
@@ -90,14 +88,15 @@ def browse_file(_, title):
 def browse_files(_, title):
     """Ask the user to select files.  Return full paths"""
     try:
+        # The File parameter is a hack to increase the buffer length.
         ret = win32gui.GetOpenFileNameW(None,
+                                        File='\x00' * 10240,
                                         Flags=win32con.OFN_ALLOWMULTISELECT
                                         | win32con.OFN_EXPLORER
                                         | win32con.OFN_FILEMUSTEXIST
                                         | win32con.OFN_HIDEREADONLY,
                                         Title=title)
-    except pywintypes.error, e:
-        logger = logging.getLogger(__name__)
+    except pywintypes.error as e:
         if 0 == e.winerror:
             logger.debug('browse_files(): user cancelled')
         else:
@@ -114,24 +113,30 @@ def browse_files(_, title):
     return pathnames
 
 
-def browse_folder(hwnd, title):
+def browse_folder(_, title):
     """Ask the user to select a folder.  Return full path."""
-    pidl = shell.SHBrowseForFolder(hwnd, None, title)[0]
-    if None == pidl:
+    pidl = shell.SHBrowseForFolder(None, None, title)[0]
+    if pidl is None:
         # user cancelled
         return None
-    fullpath = shell.SHGetPathFromIDList(pidl)
+    fullpath = shell.SHGetPathFromIDListW(pidl)
     return fullpath
 
 
+def cleanup_nonce():
+    """On exit, clean up GTK junk files"""
+    for fn in glob.glob(os.path.expandvars('%TEMP%\gdbus-nonce-file-*')):
+        logger.debug('cleaning GTK nonce file: %s', fn)
+        FileUtilities.delete(fn)
+
+
 def csidl_to_environ(varname, csidl):
     """Define an environment variable from a CSIDL for use in CleanerML and Winapp2.ini"""
-    logger = logging.getLogger(__name__)
     try:
         sppath = shell.SHGetSpecialFolderPath(None, csidl)
     except:
         logger.info(
-            'exception when getting special folder path for %s' % varname)
+            'exception when getting special folder path for %s', varname)
         return
     # there is exception handling in set_environ()
     set_environ(varname, sppath)
@@ -140,21 +145,10 @@ def csidl_to_environ(varname, csidl):
 def delete_locked_file(pathname):
     """Delete a file that is currently in use"""
     if os.path.exists(pathname):
-        try:
-            win32api.MoveFileEx(
-                pathname, None, win32con.MOVEFILE_DELAY_UNTIL_REBOOT)
-        except pywintypes.error, e:
-            if not 5 == e.winerror:
-                raise e
-            if shell.IsUserAnAdmin():
-                logger = logging.getLogger(__name__)
-                logger.warning(
-                    'Unable to queue locked file for deletion, even with administrator rights: %s' % pathname)
-                return
-            # show more useful message than "error: (5, 'MoveFileEx', 'Access
-            # is denied.')"
-            raise RuntimeError(
-                'Access denied when attempting to delete locked file without administrator rights: %s' % pathname)
+        MOVEFILE_DELAY_UNTIL_REBOOT = 4
+        if 0 == windll.kernel32.MoveFileExW(pathname, None, MOVEFILE_DELAY_UNTIL_REBOOT):
+            from ctypes import WinError
+            raise WinError()
 
 
 def delete_registry_value(key, value_name, really_delete):
@@ -165,9 +159,9 @@ def delete_registry_value(key, value_name, really_delete):
     (hive, sub_key) = split_registry_key(key)
     if really_delete:
         try:
-            hkey = _winreg.OpenKey(hive, sub_key, 0, _winreg.KEY_SET_VALUE)
-            _winreg.DeleteValue(hkey, value_name)
-        except WindowsError, e:
+            hkey = winreg.OpenKey(hive, sub_key, 0, winreg.KEY_SET_VALUE)
+            winreg.DeleteValue(hkey, value_name)
+        except WindowsError as e:
             if e.winerror == 2:
                 # 2 = 'file not found' means value does not exist
                 return False
@@ -175,15 +169,14 @@ def delete_registry_value(key, value_name, really_delete):
         else:
             return True
     try:
-        hkey = _winreg.OpenKey(hive, sub_key)
-        _winreg.QueryValueEx(hkey, value_name)
-    except WindowsError, e:
+        hkey = winreg.OpenKey(hive, sub_key)
+        winreg.QueryValueEx(hkey, value_name)
+    except WindowsError as e:
         if e.winerror == 2:
             return False
         raise
     else:
         return True
-    raise RuntimeError('Unknown error in delete_registry_value')
 
 
 def delete_registry_key(parent_key, really_delete):
@@ -195,8 +188,8 @@ def delete_registry_key(parent_key, really_delete):
     (hive, parent_sub_key) = split_registry_key(parent_key)
     hkey = None
     try:
-        hkey = _winreg.OpenKey(hive, parent_sub_key)
-    except WindowsError, e:
+        hkey = winreg.OpenKey(hive, parent_sub_key)
+    except WindowsError as e:
         if e.winerror == 2:
             # 2 = 'file not found' happens when key does not exist
             return False
@@ -205,13 +198,13 @@ def delete_registry_key(parent_key, really_delete):
     if not hkey:
         # key not found
         return False
-    keys_size = _winreg.QueryInfoKey(hkey)[0]
+    keys_size = winreg.QueryInfoKey(hkey)[0]
     child_keys = []
     for i in range(keys_size):
-        child_keys.append(parent_key + '\\' + _winreg.EnumKey(hkey, i))
+        child_keys.append(parent_key + '\\' + winreg.EnumKey(hkey, i))
     for child_key in child_keys:
         delete_registry_key(child_key, True)
-    _winreg.DeleteKey(hive, parent_sub_key)
+    winreg.DeleteKey(hive, parent_sub_key)
     return True
 
 
@@ -250,12 +243,15 @@ def delete_updates():
 
 def detect_registry_key(parent_key):
     """Detect whether registry key exists"""
-    parent_key = str(parent_key)  # Unicode to byte string
+    try:
+        parent_key = str(parent_key)  # Unicode to byte string
+    except UnicodeEncodeError:
+        return False
     (hive, parent_sub_key) = split_registry_key(parent_key)
     hkey = None
     try:
-        hkey = _winreg.OpenKey(hive, parent_sub_key)
-    except WindowsError, e:
+        hkey = winreg.OpenKey(hive, parent_sub_key)
+    except WindowsError as e:
         if e.winerror == 2:
             # 2 = 'file not found' happens when key does not exist
             return False
@@ -265,41 +261,37 @@ def detect_registry_key(parent_key):
     return True
 
 
-def elevate_privileges():
+def elevate_privileges(uac):
     """On Windows Vista and later, try to get administrator
     privileges.  If successful, return True (so original process
     can exit).  If failed or not applicable, return False."""
 
-    if parse_windows_build() < 6:
-        # Windows XP does not have the UAC.
-        # Vista is the first version Windows that has the UAC.
-        # 5.1 = Windows XP
-        # 6.0 = Vista
-        # 6.1 = 7
-        # 6.2 = 8
-        # 10 = 10
-        return False
-
-    logger = logging.getLogger(__name__)
-
     if shell.IsUserAnAdmin():
         logger.debug('already an admin (UAC not required)')
+        htoken = win32security.OpenProcessToken(
+            win32api.GetCurrentProcess(), win32security.TOKEN_ADJUST_PRIVILEGES | win32security.TOKEN_QUERY)
+        newPrivileges = [
+            (win32security.LookupPrivilegeValue(None, "SeBackupPrivilege"), win32security.SE_PRIVILEGE_ENABLED),
+            (win32security.LookupPrivilegeValue(None, "SeRestorePrivilege"), win32security.SE_PRIVILEGE_ENABLED),
+        ]
+        win32security.AdjustTokenPrivileges(htoken, 0, newPrivileges)
+        win32file.CloseHandle(htoken)
+        return False
+    elif not uac:
         return False
 
     if hasattr(sys, 'frozen'):
         # running frozen in py2exe
-        exe = unicode(sys.executable, sys.getfilesystemencoding())
+        exe = sys.executable
         parameters = "--gui --no-uac"
     else:
-        # __file__ is absolute path to bleachbit/Windows.py
-        pydir = os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))
-        pyfile = os.path.join(pydir, 'GUI.py')
+        pyfile = os.path.join(bleachbit.bleachbit_exe_path, 'bleachbit.py')
         # If the Python file is on a network drive, do not offer the UAC because
         # the administrator may not have privileges and user will not be
         # prompted.
         if len(pyfile) > 0 and path_on_network(pyfile):
             logger.debug(
-                "debug: skipping UAC because '%s' is on network" % pyfile)
+                "debug: skipping UAC because '%s' is on network", pyfile)
             return False
         parameters = '"%s" --gui --no-uac' % pyfile
         exe = sys.executable
@@ -307,8 +299,7 @@ def elevate_privileges():
     # add any command line parameters such as --debug-log
     parameters = "%s %s" % (parameters, ' '.join(sys.argv[1:]))
 
-    logger.debug('elevate_privileges() exe=%s, parameters=%s' %
-                 (exe, parameters))
+    logger.debug('elevate_privileges() exe=%s, parameters=%s', exe, parameters)
 
     rc = None
     try:
@@ -316,13 +307,13 @@ def elevate_privileges():
                                   lpFile=exe,
                                   lpParameters=parameters,
                                   nShow=win32con.SW_SHOW)
-    except pywintypes.error, e:
+    except pywintypes.error as e:
         if 1223 == e.winerror:
             logger.debug('user denied the UAC dialog')
             return False
         raise
 
-    logger.debug('ShellExecuteEx=%s' % rc)
+    logger.debug('ShellExecuteEx=%s', rc)
 
     if isinstance(rc, dict):
         return True
@@ -348,30 +339,18 @@ def empty_recycle_bin(path, really_delete):
     return bytes_used
 
 
-def get_autostart_path():
-    """Return the path of the BleachBit shortcut in the user's startup folder"""
+def get_clipboard_paths():
+    """Return a tuple of Unicode pathnames from the clipboard"""
+    import win32clipboard
+    win32clipboard.OpenClipboard()
+    path_list = ()
     try:
-        startupdir = shell.SHGetSpecialFolderPath(None, shellcon.CSIDL_STARTUP)
-    except:
-        # example of failure
-        # http://www.bleachbit.org/forum/error-windows-7-x64-bleachbit-091
-        logger = logging.getLogger(__name__)
-        logger.exception('exception in get_autostart_path()')
-        msg = 'Error finding user startup folder: %s ' % (
-            str(sys.exc_info()[1]))
-        import GuiBasic
-        GuiBasic.message_dialog(None, msg)
-        # as a fallback, guess
-        # Windows XP: C:\Documents and Settings\(username)\Start Menu\Programs\Startup
-        # Windows 7:
-        # C:\Users\(username)\AppData\Roaming\Microsoft\Windows\Start
-        # Menu\Programs\Startup
-        startupdir = os.path.expandvars(
-            '$USERPROFILE\\Start Menu\\Programs\\Startup')
-        if not os.path.exists(startupdir):
-            startupdir = os.path.expandvars(
-                '$APPDATA\\Roaming\\Microsoft\\Windows\\Start Menu\\Programs\\Startup')
-    return os.path.join(startupdir, 'bleachbit.lnk')
+        path_list = win32clipboard.GetClipboardData(win32clipboard.CF_HDROP)
+    except TypeError:
+        pass
+    finally:
+        win32clipboard.CloseClipboard()
+    return path_list
 
 
 def get_fixed_drives():
@@ -456,11 +435,11 @@ def get_recycle_bin():
     for item in h:
         path = h.GetDisplayNameOf(item, shellcon.SHGDN_FORPARSING)
         if os.path.isdir(path):
+            # Return the contents of a normal directory, but do
+            # not recurse Windows symlinks in the Recycle Bin.
             for child in FileUtilities.children_in_directory(path, True):
                 yield child
-            yield path
-        else:
-            yield path
+        yield path
 
 
 def get_windows_version():
@@ -470,77 +449,35 @@ def get_windows_version():
     return Decimal(vstr)
 
 
+def is_junction(path):
+    """Check whether the path is a link
+
+    On Python 2.7 the function os.path.islink() always returns False,
+    so this is needed
+    """
+    FILE_ATTRIBUTE_REPARSE_POINT = 0x400
+    attr = windll.kernel32.GetFileAttributesW(path)
+    return bool(attr & FILE_ATTRIBUTE_REPARSE_POINT)
+
+
+
 def is_process_running(name):
-    """Return boolean whether process (like firefox.exe) is running"""
-
-    if os.getenv('PROGRAMFILES(X86)'):
-        # 64-bit Windows detected
-        return is_process_running_wmic(name)
-    else:
-        # 32-bit Windows detected
-        return is_process_running_win32(name)
-
-
-def is_process_running_win32(name):
     """Return boolean whether process (like firefox.exe) is running
 
-    Does not work on 64-bit Windows
-
-    Originally by Eric Koome
-    license GPL
-    http://code.activestate.com/recipes/305279/
+    Works on Windows Vista or later, but on Windows XP gives an ImportError
     """
 
-    hModule = c_ulong()
-    count = c_ulong()
-    modname = c_buffer(30)
-    PROCESS_QUERY_INFORMATION = 0x0400
-    PROCESS_VM_READ = 0x0010
-
-    for pid in win32process.EnumProcesses():
-
-        # Get handle to the process based on PID
-        hProcess = kernel.OpenProcess(
-            PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
-            False, pid)
-        if hProcess:
-            psapi.EnumProcessModules(
-                hProcess, byref(hModule), sizeof(hModule), byref(count))
-            psapi.GetModuleBaseNameA(
-                hProcess, hModule.value, modname, sizeof(modname))
-            clean_modname = "".join(
-                [i for i in modname if i != '\x00']).lower()
-
-            # Clean up
-            for i in range(modname._length_):
-                modname[i] = '\x00'
-
-            kernel.CloseHandle(hProcess)
-
-            if len(clean_modname) > 0 and '?' != clean_modname:
-                # Filter out non-ASCII characters which we don't need
-                # and which may cause display warnings
-                clean_modname2 = re.sub(
-                    r'[^a-z\.]', '_', clean_modname.lower())
-                if clean_modname2 == name.lower():
-                    return True
-
+    import psutil
+    name = name.lower()
+    for proc in psutil.process_iter():
+        try:
+            if proc.name().lower() == name:
+                return True
+        except psutil.NoSuchProcess:
+            pass
     return False
 
 
-def is_process_running_wmic(name):
-    """Return boolean whether process (like firefox.exe) is running
-
-    Works on Windows XP Professional but not on XP Home
-    """
-
-    clean_name = re.sub(r'[^A-Za-z\.]', '_', name).lower()
-    args = ['wmic', 'path', 'win32_process', 'where', "caption='%s'" %
-            clean_name, 'get', 'Caption']
-    (_, stdout, _) = General.run_external(args)
-    return stdout.lower().find(clean_name) > -1
-
-
 def move_to_recycle_bin(path):
     """Move 'path' into recycle bin"""
     shell.SHFileOperation(
@@ -577,26 +514,23 @@ def shell_change_notify():
 
 def set_environ(varname, path):
     """Define an environment variable for use in CleanerML and Winapp2.ini"""
-    logger = logging.getLogger(__name__)
     if not path:
-        logger.debug('set_environ(%s, %s): skipping because blank path' %
-                     (varname, path))
-        # Such as LocalAppDataLow on XP
         return
-    if os.environ.has_key(varname):
-        logger.debug('set_environ(%s, %s): skipping because environment variable is already defined' %
-                     (varname, path))
+    if varname in os.environ:
+        #logger.debug('set_environ(%s, %s): skipping because environment variable is already defined', varname, path)
+        if 'nt' == os.name:
+            os.environ[varname] = os.path.expandvars('%%%s%%' % varname)
         # Do not redefine the environment variable when it already exists
+        # But re-encode them with utf-8 instead of mbcs
         return
     try:
         if not os.path.exists(path):
             raise RuntimeError(
                 'Variable %s points to a non-existent path %s' % (varname, path))
         os.environ[varname] = path
-        logger.debug('set_environ(%s, %s), set' % (varname, path))
     except:
         logger.exception(
-            'set_environ(%s, %s): exception when setting environment variable' % (varname, path))
+            'set_environ(%s, %s): exception when setting environment variable', varname, path)
 
 
 def setup_environment():
@@ -614,10 +548,13 @@ def setup_environment():
     try:
         path = get_known_folder_path('LocalAppDataLow')
     except:
-        logger = logging.getLogger(__name__)
         logger.exception('exception identifying LocalAppDataLow')
     else:
         set_environ('LocalAppDataLow', path)
+    # %cd% can be helpful for cleaning portable applications when
+    # BleachBit is portable. It is the same variable name as defined by
+    # cmd.exe .
+    set_environ('cd', os.getcwd())
 
 
 def split_registry_key(full_key):
@@ -626,33 +563,10 @@ def split_registry_key(full_key):
     assert len(full_key) >= 6
     [k1, k2] = full_key.split("\\", 1)
     hive_map = {
-        'HKCR': _winreg.HKEY_CLASSES_ROOT,
-        'HKCU': _winreg.HKEY_CURRENT_USER,
-        'HKLM': _winreg.HKEY_LOCAL_MACHINE,
-        'HKU': _winreg.HKEY_USERS}
+        'HKCR': winreg.HKEY_CLASSES_ROOT,
+        'HKCU': winreg.HKEY_CURRENT_USER,
+        'HKLM': winreg.HKEY_LOCAL_MACHINE,
+        'HKU': winreg.HKEY_USERS}
     if k1 not in hive_map:
         raise RuntimeError("Invalid Windows registry hive '%s'" % k1)
-    return (hive_map[k1], k2)
-
-
-def start_with_computer(enabled):
-    """If enabled, create shortcut to start application with computer.
-    If disabled, then delete the shortcut."""
-    autostart_path = get_autostart_path()
-    if not enabled:
-        if os.path.lexists(autostart_path):
-            FileUtilities.delete(autostart_path)
-        return
-    if os.path.lexists(autostart_path):
-        return
-    import win32com.client
-    wscript_shell = win32com.client.Dispatch('WScript.Shell')
-    shortcut = wscript_shell.CreateShortCut(autostart_path)
-    shortcut.TargetPath = os.path.join(
-        Common.bleachbit_exe_path, 'bleachbit.exe')
-    shortcut.save()
-
-
-def start_with_computer_check():
-    """Return boolean whether BleachBit will start with the computer"""
-    return os.path.lexists(get_autostart_path())
+    return hive_map[k1], k2
